require_relative '../filelogger'

class Index
  include Singleton
  def initialize()
    @index = 0
  end
  def increase()
    @index = @index + 1
  end
  def get()
    return @index
  end
end

module Embulk
  module Parser

    class BinaryParserPlugin < ParserPlugin
      Plugin.register_parser("binary", self)

      CHUNK_SIZE = 50
      MEGA = 1000000
      EOF = "EOF"

      def self.transaction(config, &control)
	tool_dir = config.param('tool_dir', :string, default: ".")
	@logger = LogManager.instance()
	@logger.setFilename(tool_dir, "parser")
        task = {
	  chunk_size: config.param('chunk_size', :float, default: CHUNK_SIZE) * MEGA,
	  supplier: config.param("supplier", :string, default: "parser"),
	  data_set: config.param("data_set", :string),
	  input_plugin: config.param("storage", :string, default: "parser"),
	  date: Time.now.strftime("%Y-%m-%d_%H-%M-%S")
        }
	columns = [
            Column.new(0, "supplier", :string),
            Column.new(1, "data_set", :string),
	    Column.new(2, "file", :string),
	    Column.new(3, "extension", :string),
	    Column.new(4, "data_chunk", :string),
	    Column.new(5, "eof", :string),
	    Column.new(6, "size", :string),
	    Column.new(7, "hash", :string)
          ]

        yield(task, columns)
      end

      def run(file_input)
        @index = Index.instance().get()
	@logger = LogManager.instance()
        while file = file_input.next_file
	  begin
	    filename = "file_from_#{task['input_plugin']}_#{task['date']}"
	    each_chunk(file, filename, task['chunk_size']) do |record|
              @page_builder.add(record)
            end
            @page_builder.finish
	    Index.instance().increase()
          rescue java.lang.OutOfMemoryError
	    @logger.logOutOfMemoryError(path)
	    return
	  rescue Exception => e
	    @logger.error("An error occurred during file ingestion: " + e.to_s, print=TRUE)
	    @logger.error(e.backtrace)
	    puts "[INFO] For more detailed information, please refer to the log file: " + @logger.getLogPath()
	  end
        end
      end

      private
      def each_chunk(file, filename, chunk_size=CHUNK_SIZE)
	extension = @index.to_s.rjust(3, "0")
	npart = 0
        next_byte = file.read(1)
	first = TRUE
        while true
            data = next_byte
            if not next_byte
		if first
		  # this means this is an empty file
		  values = [task['supplier'], task['data_set'], filename, extension, "", "", "", ""]
		  yield(values)
		end
		break
	    end
            data += file.read(chunk_size)
            next_byte = file.read(1)
            if not next_byte
                eof = EOF
		if first
		  # this means that the whole file will be ingested at once (not split)
		  eof = ""
		end
	    else
		npart += 1
       	 	eof = npart.to_s.rjust(3, "0")
	    end
	    content = Base64.encode64(data)
	    values = [task['supplier'], task['data_set'], filename, extension, content, eof, "", ""]
	    first = FALSE
	    yield(values)
	end
      end

    end
  end
end
